/* fft.c : for xlevel 
 * programmed by K. Takebe, 1997/11/02
 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "xlevel.h"

static int  order = 0;
static int  dnum  = 0;
int        *bitrev_table;
double     *cos_table;
double     *sin_table;


int init_fft_param(int n)
{
  int i, j, k;

  order = 0;
  i     = n;
  while ((i /= 2) != 0) order++;

  j = 1;
  for (i = 0; i < order; i++) j *= 2;
  dnum = j;

  cos_table    = (double*)calloc(3*dnum/4, sizeof(double));
  bitrev_table =   (int *)calloc(dnum,     sizeof(int));
  if ( cos_table == NULL || bitrev_table == NULL ) {
    perror("");
    exit(1);
  }

  sin_table = cos_table + dnum/4;
  for ( i = 0; i < 3*dnum/4; i++ )
    cos_table[i] = cos(2.0*M_PI/dnum*(double)i);


  k = dnum / 2;
  j = 1;
  bitrev_table[0] = 0;
  
  while (k >= 1) {
    for (i = 0; i < j; i++)
      bitrev_table[i+j] = bitrev_table[i] + k;
    k /= 2;
    j *= 2;
  }

  return j;
}


int fft1(double *ar, double *ai)
{
  int     i, j, it, xp, xp2, k, j1, j2;
  double  wr, wi, tr, ti;
  int     m;

  xp2 = dnum;
  for ( it = 0; it < order; it ++ ) {
    xp  = xp2;
    xp2 = xp / 2;
    for ( k = 0; k < xp2; k ++ ) {
      m  = k * dnum / xp;
      wr = cos_table[m];
      wi = sin_table[m];

      i = k - xp;
      for ( j = xp; j <= dnum; j += xp ) {
	/* Butterfly processing */

	j1 = j  + i;
	j2 = j1 + xp2;

	tr   = ar[j1] - ar[j2];
	ti   = ai[j1] - ai[j2];

	ar[j1] = ar[j1] + ar[j2];
	ai[j1] = ai[j1] + ai[j2];

	ar[j2] = tr * wr - ti * wi;
	ai[j2] = ti * wr + tr * wi;
      }
    }
  }

  /* bit reverse */
  for ( i = 0; i < dnum; i++ ) {
    j = bitrev_table[i];
    if ( i < j ) {
      tr = ar[j], ar[j] = ar[i], ar[i] = tr;
      ti = ai[j], ai[j] = ai[i], ai[i] = ti;
    }
  }
  return 0;
}
