/* mixing.c : for xlevel 
 * programmed by K. Takebe, 1997/11/02
 * Referenced:  Linux Multimedia Guide (Jeff Tranter; O'Reilly)
 */
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#if   defined(linux)
#include <linux/soundcard.h>
#elif defined(__FreeBSD__)
#include <machine/soundcard.h>
#endif

#include "xlevel.h"


static int   fd = -1;     /* file descriptor for mixer device */

int open_mixer_device(char *mixer_device)
{
  if ( mixer_device == NULL ) {
    fprintf(stderr, "open_mixer_device(): device name is NULL. ");
    return 1;
  }

  /* open mixer, read only */
  fd = open(mixer_device, O_RDONLY);
  if (fd == -1) {
    fprintf(stderr, "open_mixer_device(): unable to open %s. ", mixer_device);
    perror("");
    return 1;
  }

  if (debug || verbose) {
    printf("\tmixer device file  = %s\n", mixer_device);
  }
  return 0;
}

int close_mixer_device()
{
  if ( fd != -1 )
    close(fd);
  fd = -1;
  return 0;
}


int mixer_setting(char *mixer_device, char *rec_devices)
{
  int   i;      /* loop counter */
  int   status;   /* status of system calls */
  /* various device settings */
  int   recsrc, devmask, recmask, stereodevs;
  /* names of available mixer channels */
  char *sound_device_names[] = SOUND_DEVICE_LABELS;

  int  pm;
  char splits[80], *p;

  strcpy(splits, rec_devices);

  open_mixer_device(mixer_device);

  /* get all of the information about the mixer */
  status = ioctl(fd, SOUND_MIXER_READ_RECSRC, &recsrc);
  if (status == -1)
    perror("SOUND_MIXER_READ_RECSRC ioctl failed");
  status = ioctl(fd, SOUND_MIXER_READ_DEVMASK, &devmask);
  if (status == -1)
    perror("SOUND_MIXER_READ_DEVMASK ioctl failed");
  status = ioctl(fd, SOUND_MIXER_READ_RECMASK, &recmask);
  if (status == -1)
    perror("SOUND_MIXER_READ_RECMASK ioctl failed");
  status = ioctl(fd, SOUND_MIXER_READ_STEREODEVS, &stereodevs);
  if (status == -1)
    perror("SOUND_MIXER_READ_STEREODEVS ioctl failed");

  p = strtok(splits, ",");
  if ( p != NULL ) {
    if ( strcmp(p, "all") == 0 ) {
      status = ioctl(fd, SOUND_MIXER_WRITE_RECSRC, &recmask);
      if (status == -1)
	perror("SOUND_MIXER_WRITE_RECSRC ioctl failed");
    } else if ( strcmp(p, "none") != 0 ) {
      pm = 0;
      while ( p != NULL ) {
	for (i = 0 ; i < SOUND_MIXER_NRDEVICES ; i++)
	  if ( strncmp(p, sound_device_names[i], strlen(p)) == 0 &&
	      ((1 << i) & recmask) ) {
	    pm = pm | (1<<i);
	    break;
	  } 
	p = strtok(NULL, ",");
      }
      status = ioctl(fd, SOUND_MIXER_WRITE_RECSRC, &pm );
      if (status == -1)
	perror("SOUND_MIXER_WRITE_RECSRC ioctl failed");
      recsrc = pm;
    }
    if (debug||verbose) {
      char devices[80], *s, *p;
      s = devices;
      for (i = 0 ; i < SOUND_MIXER_NRDEVICES ; i++)
	if ( (1<<i) & recsrc ) {
	  p = sound_device_names[i];
	  while ( *p != ' ' && *p != '\0' ) {
	    *s = *p;  s++; p++;
	  }
	  *s = ','; s++;
	}
      s--;
      *s = '\0';
      printf("\ttarget instruments = %s\n", devices);
    }      
  }

  close_mixer_device();
  return 0;
}


int check_available_devices(char *mixer_device, char *devices_list)
{
  int   i;        /* loop counter */
  int   status;   /* status of system calls */
  int   recmask;
  char *sound_device_names[] = SOUND_DEVICE_LABELS;

  char *p, *s;

  open_mixer_device(mixer_device);

  status = ioctl(fd, SOUND_MIXER_READ_RECMASK, &recmask);
  if (status == -1)
    perror("SOUND_MIXER_READ_RECMASK ioctl failed");

  s = devices_list;
  for (i = 0 ; i < SOUND_MIXER_NRDEVICES ; i++)
    if ( (1<<i) & recmask ) {
      p = sound_device_names[i];
      while ( *p != ' ' && *p != '\0' ) {
	*s = *p;  s++; p++;
      }
      *s = ','; s++;
    }
  s--;
  *s = '\0';

  close_mixer_device();
  return status;
}
